@@ -5,10 +5,10 @@ nox.options.reuse_existing_virtualenvs = True
 
 
 @nox.session(python=["3.8", "3.9", "3.10", "3.11", "3.12", "3.13", "3.14"])
-def tests(session):
+def tests(session: nox.Session) -> None:
     session.install(".", "pytest")
     session.run("pytest")
 
 
 if __name__ == "__main__":
-     nox.main()
+    nox.main()
