@@ -1,107 +1,143 @@
-from __future__ import print_function, division
-
-import sys
+import builtins
+import decimal
+import functools
 import inspect
 import math
-import decimal
 import operator
-import functools
+import sys
 from pprint import pprint
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    TextIO,
+    Tuple,
+    Union,
+)
 
 from engineering_notation import EngNumber
+from rich.console import Console
 
-try:
-    import builtins
-except ImportError:
-    if sys.version_info < (3,):
-        print('The calculator only runs under Python 3.', file=sys.stderr)
-        sys.exit(1)
-    else:
-        raise
-
-from rpnpy.errors import StackError
-from rpnpy.functions import addSpecialFunctions
+from rpnpy.errors import (
+    CalculatorError,
+    IncompatibleModifiersError,
+    StackError,
+    UnknownModifiersError,
+)
+from rpnpy.functions import FUNCTIONS
 from rpnpy.inspect import countArgs
 from rpnpy.io import findCommands
-from rpnpy.errors import (
-    UnknownModifiersError, IncompatibleModifiersError, CalculatorError)
+from rpnpy.modifiers import Modifiers
+from rpnpy.utils import plural
 
 
 class Function:
-    def __init__(self, moduleName, name, func, nArgs):
+    def __init__(
+        self, moduleName: str, name: str, func: Callable, nArgs: Optional[int]
+    ) -> None:
         self.moduleName = moduleName
         self.name = name
         self.func = func
         self.nArgs = nArgs
 
-    def __call__(self, *args, **kw):
+    def __call__(self, *args: Any, **kw: Any) -> Any:
         return self.func(*args, **kw)
 
-    def __repr__(self):
-        return 'Function(%s (calls %s with %d arg%s))' % (
-            self.name, self.path, self.nArgs, '' if self.nArgs == 1 else 's')
+    def __repr__(self) -> str:
+        assert self.nArgs is not None
+        return (
+            f"Function({self.name} (calls {self.path} with {self.nArgs} "
+            f"arg{plural(self.nArgs)}))"
+        )
 
     @property
-    def path(self):
-        return '%s.%s' % (self.moduleName, self.name)
+    def path(self) -> str:
+        return f"{self.moduleName}.{self.name}"
 
 
 class Variable:
-    def __init__(self, name, variables):
+    def __init__(self, name: str, variables: Dict[str, Any]) -> None:
         self.name = name
         self._variables = variables
 
-    def __repr__(self):
-        return 'Variable(%s, current value: %r)' % (
-            self.name, self._variables[self.name])
+    def __repr__(self) -> str:
+        name = self.name
+        return f"Variable({name}, current value: {self._variables[name]}!r)"
 
 
 class Calculator:
-
-    OVERRIDES = set('builtins.list builtins.map builtins.quit '
-                    'functools.reduce'.split())
+    OVERRIDES = set("builtins.list builtins.map builtins.quit functools.reduce".split())
     # Sentinel value for calculator functions to return to indicate that they
     # did not result in a value that can/should be printed.
     NO_VALUE = object()
 
-    def __init__(self, autoPrint=False, splitLines=True, separator=None,
-                 outfp=sys.stdout, errfp=sys.stderr, debug=False):
+    def __init__(
+        self,
+        autoPrint: bool = False,
+        splitLines: bool = True,
+        separator: Optional[str] = None,
+        outfp: TextIO = sys.stdout,
+        errfp: TextIO = sys.stderr,
+        debug: bool = False,
+        color: bool = False,
+    ) -> None:
         self._autoPrint = autoPrint
         self._splitLines = splitLines
         self._separator = separator
         self._outfp = outfp
         self._errfp = errfp
         self._debug = debug
+        if color:
+            self._console = Console()
+            self._errorConsole = Console(stderr=True)
+        else:
+            self._console = self._errorConsole = None
         self.stack = []
         self._previousStack = self._previousVariables = None
         self._functions = {}
         self._special = {}
-        self._variables = {}
+        self._variables = {
+            "self": self,
+        }
 
         self.addSpecialCases()
-        addSpecialFunctions(self)
-        for module in math, operator, builtins, functools, decimal:
-            self.importCallables(module)
+        self.addFunctions()
+        self.addModules()
         self.addAbbrevs()
         self.addConstants()
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.stack)
 
-    def report(self, *args, **kw):
-        print(*args, file=self._outfp, **kw)
+    def report(self, *args: Any, **kw: Any) -> None:
+        if self._console:
+            self._console.print(*args, style="bold green", **kw)
+        else:
+            print(*args, file=self._outfp, **kw)
 
-    def err(self, *args, **kw):
-        print(*args, file=self._errfp, **kw)
+    def err(self, *args: Any, **kw: Any) -> None:
+        if self._errorConsole:
+            self._errorConsole.print(*args, style="bold red", **kw)
+        else:
+            print(*args, file=self._errfp, **kw)
 
-    def debug(self, *args, **kw):
+    def debug(self, *args: Any, **kw: Any) -> None:
         if self._debug:
-            print('      #', *args, file=self._errfp, **kw)
+            if self._console:
+                self._console.print("      #", *args, style="bold yellow", **kw)
+            else:
+                print("      #", *args, file=self._errfp, **kw)
 
-    def pprint(self, *args, **kw):
-        pprint(*args, stream=self._outfp, **kw)
+    def pprint(self, *args: Any, **kw: Any) -> None:
+        if self._console:
+            self._console.print(*args, **kw)
+        else:
+            pprint(*args, stream=self._outfp, **kw)
 
-    def batch(self, fp, finalPrint=False):
+    def batch(self, fp: Iterable[str], finalPrint: bool = False) -> None:
         """Non-interactively read and execute commands from a file.
 
         @param fp: An iterator to read commands from.
@@ -116,7 +152,7 @@ class Calculator:
         if finalPrint and self.stack:
             self.printStack(-1 if len(self) == 1 else None)
 
-    def repl(self, prompt):
+    def repl(self, prompt: str) -> None:
         """Interactive read-eval-print loop.
 
         @param prompt: The C{str} prompt to print at the start of each line.
@@ -126,7 +162,7 @@ class Calculator:
                 try:
                     line = input(prompt)
                 except ValueError as e:
-                    if str(e) == 'I/O operation on closed file.':
+                    if str(e) == "I/O operation on closed file.":
                         # The user may have typed 'quit()'.
                         self.report()
                         break
@@ -140,43 +176,49 @@ class Calculator:
                 self.report()
                 break
 
-    def registerSpecial(self, func, name):
-        self._special[name] = func
-
-    def register(self, func, name=None, nArgs=None, moduleName=None):
+    def register(
+        self,
+        func: Callable,
+        name: Optional[str] = None,
+        nArgs: Optional[int] = None,
+        moduleName: Optional[str] = None,
+    ) -> None:
         name = name or func.__name__
         if name in self._functions:
-            self.debug('Registering new functionality for already known '
-                       'function named %r.' % name)
+            self.debug(
+                "Registering new functionality for already known "
+                "function named %r." % name
+            )
 
-        moduleName = moduleName or 'calculator-registered-method'
+        moduleName = moduleName or "calculator-registered-method"
         nArgs = countArgs(func, 1) if nArgs is None else nArgs
         self._functions[name] = Function(moduleName, name, func, nArgs)
 
-    def addAbbrevs(self):
+    def addAbbrevs(self) -> None:
         for longName, shortNames in (
-                ('math.log', ('log',)),
-                ('operator.attrgetter', ('attrgetter',)),
-                ('operator.itemgetter', ('itemgetter',)),
-                ('operator.methodcaller', ('methodcaller',)),
-                ('operator.add', ('+',)),
-                ('operator.eq', ('==',)),
-                ('operator.mul', ('*',)),
-                ('operator.ne', ('!=',)),
-                ('operator.sub', ('-',)),
-                ('operator.truediv', ('/', 'div')),
-                ('builtins.bool', ('bool',)),
-                ('builtins.int', ('int',)),
-                ('builtins.max', ('max',)),
-                ('builtins.min', ('min',)),
-                ('builtins.print', ('print',)),
-                ('builtins.range', ('range',)),
-                ('builtins.str', ('str',)),
+            ("math.log", ("log",)),
+            ("math.sqrt", ("v",)),
+            ("operator.attrgetter", ("attrgetter",)),
+            ("operator.itemgetter", ("itemgetter",)),
+            ("operator.methodcaller", ("methodcaller",)),
+            ("operator.add", ("+",)),
+            ("operator.eq", ("==",)),
+            ("operator.mul", ("*",)),
+            ("operator.ne", ("!=",)),
+            ("operator.sub", ("-",)),
+            ("operator.truediv", ("/", "div")),
+            ("builtins.bool", ("bool",)),
+            ("builtins.int", ("int",)),
+            ("builtins.max", ("max",)),
+            ("builtins.min", ("min",)),
+            ("builtins.print", ("print",)),
+            ("builtins.range", ("range",)),
+            ("builtins.str", ("str",)),
         ):
             try:
                 function = self._functions[longName]
             except KeyError:
-                self.err('Long function name %r is unknown' % longName)
+                self.err("Long function name %r is unknown" % longName)
             else:
                 for shortName in shortNames:
                     if shortName not in self._functions:
@@ -186,60 +228,75 @@ class Calculator:
                     else:
                         assert self._functions[shortName] is function
 
-    def addSpecialCases(self):
+    def addFunctions(self) -> None:
+        """
+        Add the functions from functions.py
+        """
+        for func in FUNCTIONS:
+            for name in func.names:
+                self.debug("Adding special command %r for %s" % (name, func))
+                self._special[name] = func
+
+    def addModules(self) -> None:
+        """
+        Add callable functions from various modules.
+        """
+        for module in math, operator, builtins, functools, decimal:
+            self.importCallables(module)
+
+    def addSpecialCases(self) -> None:
         """
         Add argument counts for functions that cannot have their signatures
         inspected.
         """
         for module, func, nArgs in (
-                (math, math.log, 1),
-                (builtins, builtins.bool, 1),
-                (builtins, builtins.int, 1),
-                (builtins, builtins.float, 1),
-                (builtins, builtins.max, 1),
-                (builtins, builtins.min, 1),
-                (builtins, builtins.print, 1),
-                (builtins, builtins.str, 1),
-                (builtins, builtins.range, 1),
-                (decimal, decimal.Decimal, 1),
-                (operator, operator.attrgetter, 1),
-                (operator, operator.itemgetter, 1),
-                (operator, operator.methodcaller, 1),
+            (math, math.log, 1),
+            (builtins, builtins.bool, 1),
+            (builtins, builtins.int, 1),
+            (builtins, builtins.float, 1),
+            (builtins, builtins.max, 1),
+            (builtins, builtins.min, 1),
+            (builtins, builtins.print, 1),
+            (builtins, builtins.str, 1),
+            (builtins, builtins.range, 1),
+            (decimal, decimal.Decimal, 1),
+            (operator, operator.attrgetter, 1),
+            (operator, operator.itemgetter, 1),
+            (operator, operator.methodcaller, 1),
         ):
-            longName = '%s.%s' % (module.__name__, func.__name__)
+            longName = "%s.%s" % (module.__name__, func.__name__)
             try:
                 self._functions[longName]
             except KeyError:
                 self._functions[longName] = Function(
-                    module.__name__, func.__name__, func, nArgs)
+                    module.__name__, func.__name__, func, nArgs
+                )
             else:
-                self.err('Long function name %r is already set' % longName)
+                self.err("Long function name %r is already set" % longName)
 
-    def addConstants(self):
+    def addConstants(self) -> None:
         """Add some constants (well, constant in theory) from math"""
         constants = [
-            ('e', math.e),
-            ('inf', math.inf),
-            ('nan', math.nan),
-            ('pi', math.pi)
+            ("e", math.e),
+            ("inf", math.inf),
+            ("nan", math.nan),
+            ("pi", math.pi),
+            ("tau", math.tau),
         ]
 
-        if sys.version_info > (3, 5):
-            constants.append(('tau', math.tau))
-
         for name, value in constants:
             if name in self._variables:
-                self.err('%r is already a variable!' % name)
+                self.err("%r is already a variable!" % name)
             else:
                 self._variables[name] = value
 
-    def importCallables(self, module):
+    def importCallables(self, module: Any) -> None:
         moduleName = module.__name__
-        exec('import ' + moduleName, globals(), self._variables)
+        exec("import " + moduleName, globals(), self._variables)
         callables = inspect.getmembers(module, callable)
 
         for name, func in callables:
-            if name.startswith('_'):
+            if name.startswith("_"):
                 continue
 
             try:
@@ -253,10 +310,10 @@ class Calculator:
             if nArgs is None:
                 continue
 
-            path = '%s.%s' % (moduleName, name)
+            path = f"{moduleName}.{name}"
 
             if path in self.OVERRIDES:
-                self.debug('Not importing %r' % path)
+                self.debug(f"Not importing {path!r}")
                 continue
 
             if path not in self._functions:
@@ -265,24 +322,24 @@ class Calculator:
                 # allows the user to call it and have the arguments taken
                 # from the stack (the number of arguments used can always
                 # be specified on the command line (e.g., :3)).
-                exec('self._functions["%s"] = Function("%s", "%s", %s, %d)' %
-                     (path, moduleName, name, path, nArgs))
+                exec(
+                    'self._functions["%s"] = Function("%s", "%s", %s, %d)'
+                    % (path, moduleName, name, path, nArgs)
+                )
 
             # Import the function by name to allow the user to use it in a
             # command with an explicit argument, instead of applying it to
             # whatever is on the stack.
             if name in self._variables:
-                self.debug('name %s already defined! Ignoring %s' %
-                           (name, path))
+                self.debug(f"name {name} already defined! Ignoring {path}")
             else:
-                exec('from %s import %s' % (moduleName, name), globals(),
-                     self._variables)
+                exec(f"from {moduleName} import {name}", globals(), self._variables)
                 if name not in self._variables:
-                    self.err('name %s not now defined!!!' % name)
+                    self.err(f"name {name!r} not now defined!!!")
                 assert name not in self._functions
                 self._functions[name] = self._functions[path]
 
-    def printStack(self, n=None):
+    def printStack(self, n: Optional[Union[int, slice]] = None) -> None:
         """
         Print the stack or some item(s) from it.
 
@@ -296,19 +353,27 @@ class Calculator:
                 self.pprint(self.stack[n])
             except IndexError:
                 if n == -1:
-                    self.err('Cannot print top of stack item (stack is empty)')
+                    self.err("Cannot print top of stack item (stack is empty).")
                 else:
                     self.err(
-                        'Cannot print stack item %d (stack has only %d item%s)'
-                        % (n, len(self), '' if len(self) == 1 else 's'))
+                        f"Cannot print stack item {n} (stack has only {len(self)} "
+                        f"item{plural(len(self))})."
+                    )
 
-    def saveState(self):
+    def saveState(self) -> None:
         """Save the stack and variable state."""
         self._previousStack = self.stack.copy()
         self._previousVariables = self._variables.copy()
 
-    def _finalize(self, result, modifiers, nPop=0, extend=False,
-                  repeat=1, noValue=False):
+    def _finalize(
+        self,
+        result: Any,
+        modifiers: Modifiers,
+        nPop: int = 0,
+        extend: bool = False,
+        repeat: int = 1,
+        noValue: bool = False,
+    ) -> None:
         """Process the final result of executing a command.
 
         @param result: A C{list} or C{tuple} of results to add to the stack.
@@ -339,7 +404,7 @@ class Calculator:
             for _ in range(repeat):
                 add(result)
 
-    def execute(self, line):
+    def execute(self, line: str) -> bool:
         """
         Execute a line of commands. Stop executing commands on any error.
 
@@ -352,13 +417,13 @@ class Calculator:
             try:
                 command, modifiers, count = next(commands)
             except UnknownModifiersError as e:
-                self.err('Unknown modifiers: %s' % ', '.join(e.args))
+                self.err("Unknown modifiers: %s" % ", ".join(e.args))
                 return False
             except IncompatibleModifiersError as e:
-                self.err('Incompatible modifiers: %s' % e.args[0])
+                self.err(f"Incompatible modifiers: {e.args[0]}")
                 return False
             except CalculatorError as e:
-                self.err('Incompatible modifiers: %s' % e.args[0])
+                self.err(f"Incompatible modifiers: {e.args[0]}")
                 return False
             except StopIteration:
                 break
@@ -371,13 +436,17 @@ class Calculator:
                     except StopIteration:
                         pass
                     else:
-                        self.debug('Ignoring commands from %r on due to '
-                                   'previous error' % command)
+                        self.debug(
+                            f"Ignoring commands from {command!r} on due to "
+                            "previous error."
+                        )
                     return False
 
         return True
 
-    def _executeOneCommand(self, command, modifiers, count):
+    def _executeOneCommand(
+        self, command: str, modifiers: Modifiers, count: Optional[int]
+    ) -> bool:
         """
         Execute one command.
 
@@ -388,20 +457,20 @@ class Calculator:
         """
         if modifiers.split:
             if not self._splitLines:
-                self.debug('Line splitting switched ON')
+                self.debug("Line splitting switched ON")
             self._splitLines = True
         elif modifiers.noSplit:
             if self._splitLines:
-                self.debug('Line splitting switched OFF')
+                self.debug("Line splitting switched OFF")
             self._splitLines = False
 
         if modifiers.all:
             stackLen = len(self)
             if count is not None and count != stackLen:
                 self.err(
-                    '* modifier conflicts with explicit count %d '
-                    '(stack has %d item%s)' %
-                    (count, stackLen, '' if stackLen == 1 else 's'))
+                    f"* modifier conflicts with explicit count {count} "
+                    f"(stack has {stackLen} item{plural(stackLen)})"
+                )
                 return False
 
         if modifiers.autoPrint:
@@ -411,41 +480,47 @@ class Calculator:
             self.toggleDebug()
 
         if not command:
-            self.debug('Empty command')
+            self.debug("Empty command")
             return True
 
         if count == 0:
-            self.debug('Count was zero - nothing to do!')
+            self.debug("Count was zero - nothing to do!")
             return True
 
         try:
-            for func in (self._tryFunction, self._tryVariable,
-                         self._trySpecial, self._tryEvalExec):
+            for func in (
+                self._tryFunction,
+                self._tryVariable,
+                self._trySpecial,
+                self._tryEvalExec,
+            ):
                 status, value = func(command, modifiers, count)
                 if status:
-                    if (value is not self.NO_VALUE and
-                            (modifiers.print or self._autoPrint)):
+                    if value is not self.NO_VALUE and (
+                        modifiers.print or self._autoPrint
+                    ):
                         self.pprint(value)
                     return True
             else:
-                raise CalculatorError('Could not find a way to execute %r' %
-                                      command)
+                raise CalculatorError(f"Could not find a way to execute {command!r}.")
         except (CalculatorError, StackError) as e:
             for err in e.args:
                 self.err(err)
             return False
 
-    def _tryFunction(self, command, modifiers, count):
+    def _tryFunction(
+        self, command: str, modifiers: Modifiers, count: Optional[int]
+    ) -> Tuple[bool, Any]:
         if modifiers.forceCommand:
             return False, self.NO_VALUE
 
         try:
             function = self._functions[command]
         except KeyError:
-            self.debug('%r is not a known function' % (command,))
+            self.debug(f"{command!r} is not a known function.")
             return False, self.NO_VALUE
 
-        self.debug('Found function %r' % command)
+        self.debug(f"Found function {command!r}.")
 
         if modifiers.push:
             self._finalize(function.func, modifiers)
@@ -453,18 +528,27 @@ class Calculator:
 
         return self._runFunction(command, modifiers, count, function)
 
-    def _runFunction(self, command, modifiers, count, function):
+    def _runFunction(
+        self,
+        command: str,
+        modifiers: Modifiers,
+        count: Optional[int],
+        function: Function,
+    ) -> Tuple[bool, Any]:
         "Run a Python function."
 
-        nArgs = ((len(self) if modifiers.all else function.nArgs)
-                 if count is None else count)
+        nArgs = (
+            (len(self) if modifiers.all else function.nArgs) if count is None else count
+        )
+
+        assert nArgs is not None
 
         if len(self) < nArgs:
             raise CalculatorError(
-                'Not enough args on stack! (%s needs %d arg%s, stack has '
-                '%d item%s)' %
-                (command, nArgs, '' if nArgs == 1 else 's',
-                 len(self), '' if len(self) == 1 else 's'))
+                "Not enough args on stack! (%s needs %d arg%s, stack has "
+                "%d item%s)"
+                % (command, nArgs, plural(nArgs), len(self), plural(len(self)))
+            )
 
         args = self.convertStackArgs(self.stack[-nArgs:]) if nArgs else []
 
@@ -474,31 +558,37 @@ class Calculator:
             # function instead of the last.
             args = args[::-1]
 
-        self.debug('Calling %s with %r' % (function.name, tuple(args)))
+        self.debug(f"Calling {function.name!r} with {tuple(args)!r}")
         try:
             result = function.func(*args)
         except BaseException as e:
             raise CalculatorError(
-                'Exception running %s(%s): %s' %
-                (function.name, ', '.join(map(str, args)), e))
+                "Exception running %s(%s): %s"
+                % (function.name, ", ".join(map(str, args)), e)
+            ) from e
         else:
             self._finalize(result, modifiers, nPop=nArgs)
             return True, result
 
-    def _tryVariable(self, command, modifiers, count):
+    def _tryVariable(
+        self, command: str, modifiers: Modifiers, count: Optional[int]
+    ) -> Tuple[bool, Any]:
         if modifiers.forceCommand:
             return False, self.NO_VALUE
 
         if command in self._variables:
-            self.debug('%r is a variable (value %r)' %
-                       (command, self._variables[command]))
+            self.debug(
+                f"{command!r} is a variable (value {self._variables[command]!r})"
+            )
             value = self._variables[command]
             if callable(value):
                 if not modifiers.push:
                     return self._runFunction(
-                        command, modifiers, count,
-                        Function('<stdin>', value.__name__, value,
-                                 countArgs(value, 1)))
+                        command,
+                        modifiers,
+                        count,
+                        Function("<stdin>", value.__name__, value, countArgs(value, 1)),
+                    )
             else:
                 if modifiers.push:
                     value = Variable(command, self._variables)
@@ -507,36 +597,41 @@ class Calculator:
             self._finalize([value] * count, modifiers, extend=True)
             return True, value
         else:
-            self.debug('%r is not a variable' % command)
+            self.debug(f"{command!r} is not a variable")
             return False, self.NO_VALUE
 
-    def _trySpecial(self, command, modifiers, count):
+    def _trySpecial(
+        self, command: str, modifiers: Modifiers, count: Optional[int]
+    ) -> Tuple[bool, Any]:
         if command in self._special:
             try:
                 value = self._special[command](self, modifiers, count)
             except EOFError:
                 raise
             except BaseException as e:
-                raise CalculatorError('Could not run special command %r: %s' %
-                                      (command, e))
+                raise CalculatorError(
+                    f"Could not run special command {command!r}: {e}"
+                ) from e
             return True, value
 
         if modifiers.forceCommand:
-            raise CalculatorError('Unknown special command: %s' % command)
+            raise CalculatorError(f"Unknown special command: {command!r}.")
 
         return False, self.NO_VALUE
 
-    def _tryEvalExec(self, command, modifiers, count):
+    def _tryEvalExec(
+        self, command: str, modifiers: Modifiers, count: Optional[int]
+    ) -> Tuple[bool, Any]:
         errors = []
         possibleWhiteSpace = False
         try:
             value = eval(command, globals(), self._variables)
         except BaseException as e:
             err = str(e)
-            errors.append('Could not eval(%r): %s' % (command, err))
-            if (self._splitLines and
-                    err.startswith(
-                        'unexpected EOF while parsing (<string>, line 1)')):
+            errors.append(f"Could not eval({command!r}): {err}")
+            if self._splitLines and err.startswith(
+                "unexpected EOF while parsing (<string>, line 1)"
+            ):
                 possibleWhiteSpace = True
 
             try:
@@ -546,32 +641,36 @@ class Calculator:
                     exec(command, globals(), self._variables)
                 except BaseException as e:
                     err = str(e)
-                    errors.append('Could not exec(%r): %s' % (command, err))
-                    if (not possibleWhiteSpace and self._splitLines and
-                            err.startswith(
-                                'unexpected EOF while parsing (<string>, '
-                                'line 1)')):
+                    errors.append(f"Could not exec({command!r}): {err}")
+                    if (
+                        not possibleWhiteSpace
+                        and self._splitLines
+                        and err.startswith(
+                            "unexpected EOF while parsing (<string>, line 1)"
+                        )
+                    ):
                         possibleWhiteSpace = True
 
                     if possibleWhiteSpace:
-                        errors.append('Did you accidentally include '
-                                      'whitespace in a command line?')
-                    raise CalculatorError(*errors)
+                        errors.append(
+                            "Did you accidentally include whitespace in a command line?"
+                        )
+                    raise CalculatorError(*errors) from e
                 else:
-                    self.debug('exec(%r) worked.' % command)
+                    self.debug(f"exec({command!r}) worked.")
                     return True, self.NO_VALUE
             else:
-                self.debug('EngNumber(%s) worked: %r' % (command, value))
+                self.debug(f"EngNumber({command}) worked: {value!r}")
                 count = 1 if count is None else count
                 self._finalize(value, modifiers=modifiers, repeat=count)
                 return True, value
         else:
-            self.debug('eval %s worked: %r' % (command, value))
+            self.debug(f"eval {command!r} worked: {value!r}")
             count = 1 if count is None else count
             self._finalize(value, modifiers=modifiers, repeat=count)
             return True, value
 
-    def convertStackArgs(self, args):
+    def convertStackArgs(self, args: Iterable[Any]) -> List[Any]:
         """Convert stack items to arguments for functions.
 
         @param args: An iterable of stack items.
@@ -588,48 +687,55 @@ class Calculator:
 
         return result
 
-    def toggleAutoPrint(self, newValue=None):
+    def toggleAutoPrint(self, newValue: Optional[bool] = None) -> None:
         """Turn auto printing on/off.
 
         @param newValue: A C{bool} new setting or C{None} to toggle.
         """
         if newValue is None:
             if self._autoPrint:
-                self.debug('Auto print off')
+                self.debug("Auto print off")
                 self._autoPrint = False
             else:
                 self._autoPrint = True
-                self.debug('Auto print on')
+                self.debug("Auto print on")
         else:
             if newValue:
                 self._autoPrint = True
-                self.debug('Auto print on')
+                self.debug("Auto print on")
             else:
-                self.debug('Auto print off')
+                self.debug("Auto print off")
                 self._autoPrint = False
 
-    def toggleDebug(self, newValue=None):
+    def toggleDebug(self, newValue: Optional[bool] = None) -> None:
         """Turn debug on/off.
 
         @param newValue: A C{bool} new setting or C{None} to toggle.
         """
         if newValue is None:
             if self._debug:
-                self.debug('Debug off')
+                self.debug("Debug off")
                 self._debug = False
             else:
                 self._debug = True
-                self.debug('Debug on')
+                self.debug("Debug on")
         else:
             if newValue:
                 self._debug = True
-                self.debug('Debug on')
+                self.debug("Debug on")
             else:
-                self.debug('Debug off')
+                self.debug("Debug off")
                 self._debug = False
 
-    def _findWithArgs(self, command, description, predicate, defaultArgCount,
-                      modifiers, count):
+    def _findWithArgs(
+        self,
+        command: str,
+        description: str,
+        predicate: Callable[[Any], bool],
+        defaultArgCount: Callable[[Any], int],
+        modifiers: Modifiers,
+        count: Optional[int],
+    ) -> Tuple[Any, Tuple[Any, ...]]:
         """
         Look for something (e.g., a callable function or a string) and its
         arguments on the stack.
@@ -654,29 +760,28 @@ class Calculator:
         stackLen = len(self)
 
         if stackLen < 2 or count is not None and stackLen < count + 1:
-            raise StackError('Cannot run %r (stack has only %d item%s)' %
-                             (command, stackLen, '' if stackLen == 1 else 's'))
+            raise StackError(
+                f"Cannot run {command!r} (stack has only {stackLen} "
+                f"item{plural(stackLen)})."
+            )
 
         if modifiers.reverse:
             item = self.stack[-1]
 
             if not predicate(item):
-                raise StackError('Top stack item (%r) is not %s' %
-                                 (item, description))
+                raise StackError(f"Top stack item ({item!r}) is not {description}.")
 
             if count is None:
-                count = (stackLen - 1 if modifiers.all else
-                         defaultArgCount(item))
+                count = stackLen - 1 if modifiers.all else defaultArgCount(item)
 
             nargsAvail = stackLen - 1
             if nargsAvail < count:
                 raise StackError(
-                    'Cannot run %r with %d argument%s '
-                    '(stack has only %d item%s available)' %
-                    (command, count, '' if count == 1 else 's',
-                     nargsAvail, '' if nargsAvail == 1 else 's'))
+                    f"Cannot run {command!r} with {count} argument{plural(count)} "
+                    f"(stack has only {nargsAvail} item{plural(nargsAvail)} available)."
+                )
 
-            args = self.stack[-(count + 1):-1]
+            args = self.stack[-(count + 1) : -1]
         else:
             if count is None:
                 if modifiers.all:
@@ -690,8 +795,7 @@ class Calculator:
                         else:
                             args.append(arg)
                     else:
-                        raise StackError('Could not find %s item on stack' %
-                                         description)
+                        raise StackError(f"Could not find {description} item on stack.")
 
                     item = self.stack[-(len(args) + 1)]
                     args = args[::-1]
@@ -700,42 +804,50 @@ class Calculator:
 
                 if not predicate(item):
                     raise StackError(
-                        'Cannot run %r with %d argument%s. Stack item (%r) is '
-                        'not %s' % (command, count, '' if count == 1 else 's',
-                                    item, description))
+                        f"Cannot run {command!r} with {count} argument{plural(count)}. "
+                        f"Stack item ({item!r}) is not {description}."
+                    )
 
                 args = self.stack[-count:]
 
         return item, self.convertStackArgs(args)
 
-    def findCallableAndArgs(self, command, modifiers, count):
+    def findCallableAndArgs(
+        self, command: str, modifiers: Modifiers, count: Optional[int]
+    ) -> Tuple[Callable, Tuple[Any, ...]]:
         """Look for a callable function and its arguments on the stack.
 
         @param modifier: A C{Modifiers} instance.
         @return: A 2-C{tuple} of the function and a C{tuple} of its arguments.
         """
-        def defaultArgCount(func):
+
+        def defaultArgCount(func: Callable) -> int:
             return countArgs(func, 1)
 
-        return self._findWithArgs(command, 'callable', callable,
-                                  defaultArgCount, modifiers, count)
+        return self._findWithArgs(
+            command, "callable", callable, defaultArgCount, modifiers, count
+        )
 
-    def findStringAndArgs(self, command, modifiers, count):
+    def findStringAndArgs(
+        self, command: str, modifiers: Modifiers, count: Optional[int]
+    ) -> Tuple[str, Tuple[Any, ...]]:
         """Look for a string its arguments on the stack.
 
         @param modifier: A C{Modifiers} instance.
         @return: A 2-C{tuple} of the string and a C{tuple} of its arguments.
         """
-        def predicate(x):
+
+        def predicate(x: Any) -> bool:
             return isinstance(x, str)
 
-        def defaultArgCount(x):
+        def defaultArgCount(x: Any) -> int:
             return 1
 
-        return self._findWithArgs(command, 'a string', predicate,
-                                  defaultArgCount, modifiers, count)
+        return self._findWithArgs(
+            command, "a string", predicate, defaultArgCount, modifiers, count
+        )
 
-    def setVariable(self, variable, value):
+    def setVariable(self, variable: str, value: Any) -> None:
         """
         Set the value of a variable.
 
